import 'dart:typed_data';

import 'package:flutter/cupertino.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:scanbot_sdk/barcode_parsers/barcode_scanner.dart';
import 'package:scanbot_sdk/common_data.dart';
import 'package:scanbot_sdk/sdk_utils.dart';

import 'common_data.dart';

part 'barcode_scanning_data.g.dart';

@JsonSerializable(
    nullable: true,
    createFactory: true,
    includeIfNull: false,
    explicitToJson: true)
class BarcodeScanningResult extends Result {
  List<BarcodeItem> barcodeItems;

  Uri barcodeImageURI;

  BarcodeScanningResult(this.barcodeItems);

  BarcodeScanningResult.error(String message) {
    this.operationResult = OperationResult.ERROR;
    this.message = message;
  }

  BarcodeScanningResult.canceled() {
    this.operationResult = OperationResult.CANCELED;
  }

  factory BarcodeScanningResult.fromJson(Map<String, dynamic> json) =>
      _$BarcodeScanningResultFromJson(json);

  Map<String, dynamic> toJson() => _$BarcodeScanningResultToJson(this);
}

@JsonSerializable(
    nullable: true,
    createFactory: true,
    includeIfNull: false,
    explicitToJson: true)
class BarcodeItem {
  BarcodeFormattedResult formattedResult;

  ///Scanned barcode format
  BarcodeFormat barcodeFormat;

  ///Result data from barcode
  String text;

  @JsonKey(fromJson: uintListFromString)
  ///raw data of barcode
  Uint8List rawBytes;

  BarcodeItem(this.barcodeFormat, this.text);

  factory BarcodeItem.fromJson(Map<String, dynamic> json) =>
      _$BarcodeItemFromJson(json);

  Map<String, dynamic> toJson() => _$BarcodeItemToJson(this);
}

///Available barcode formats
enum BarcodeFormat {
  AZTEC,
  CODABAR,
  CODE_39,
  CODE_93,
  CODE_128,
  DATA_MATRIX,
  EAN_8,
  EAN_13,
  ITF,
  PDF_417,
  QR_CODE,
  RSS_14,
  RSS_EXPANDED,
  UPC_A,
  UPC_E,
  MSI_PLESSEY,
  UNKNOWN
}

class PredefinedBarcodes {
  ///return list of one dimensional barcode formats
  static List<BarcodeFormat> allBarcodeTypes() {
    return [
      BarcodeFormat.AZTEC,
      BarcodeFormat.CODABAR,
      BarcodeFormat.CODE_39,
      BarcodeFormat.CODE_93,
      BarcodeFormat.CODE_128,
      BarcodeFormat.DATA_MATRIX,
      BarcodeFormat.EAN_8,
      BarcodeFormat.EAN_13,
      BarcodeFormat.ITF,
      BarcodeFormat.PDF_417,
      BarcodeFormat.QR_CODE,
      BarcodeFormat.RSS_14,
      BarcodeFormat.RSS_EXPANDED,
      BarcodeFormat.UPC_A,
      BarcodeFormat.UPC_E,
    ];
  }

  static List<BarcodeFormat> oneDimensionalBarcodes() {
    return [
      BarcodeFormat.CODABAR,
      BarcodeFormat.CODE_128,
      BarcodeFormat.CODE_39,
      BarcodeFormat.CODE_93,
      BarcodeFormat.EAN_13,
      BarcodeFormat.EAN_8,
      BarcodeFormat.ITF,
      BarcodeFormat.UPC_A,
      BarcodeFormat.RSS_14,
      BarcodeFormat.RSS_EXPANDED,
      BarcodeFormat.UPC_E
    ];
  }

  ///return list of two dimensional barcode formats
  static List<BarcodeFormat> twoDimensionalBarcodes() {
    return [
      BarcodeFormat.AZTEC,
      BarcodeFormat.DATA_MATRIX,
      BarcodeFormat.PDF_417,
      BarcodeFormat.QR_CODE
    ];
  }
}

@JsonSerializable(nullable: true, includeIfNull: false, explicitToJson: true)
class BarcodeScannerConfiguration {
  /// Background color of the top toolbar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarBackgroundColor;

  /// Color of the titles of all buttons in the top toolbar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarButtonsColor;

  /// Background color outside of the finder window.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color cameraOverlayColor;

  /// Color of the finder window's outline.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color finderLineColor;

  /// Color of the text hint under the finder window.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color finderTextHintColor;

  ///color of active buttons on bottom bar
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color bottomButtonsActiveColor;

  ///color of inactive buttons on bottom bar
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color bottomButtonsInactiveColor;

  bool cancelButtonHidden;

  /// Title of the cancel button.
  String cancelButtonTitle;

  /// Title of the button that opens the screen where the user can allow
  /// the usage of the camera by the app.
  String enableCameraButtonTitle;

  /// Text that will be displayed when the app
  /// is not allowed to use the camera, prompting the user
  /// to enable the usage of the camera.
  String enableCameraExplanationText;

  /// Text hint shown under the finder window.
  String finderTextHint;

  String flashButtonTitle;

  /// Aspect ratio of finder view.
  FinderAspectRatio finderAspectRatio;

  /// Thickness of the finder window's outline.
  int finderLineWidth;

  /// Controls whether to play a beep sound after a successful detection.
  /// Default value is TRUE.
  bool successBeepEnabled;

  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool flashEnabled;

  /// Orientation lock mode of the camera: PORTRAIT or LANDSCAPE.
  /// By default the camera orientation is not locked.

  CameraOrientationMode orientationLockMode;

  /// Format of barcode which  need to recognize
  List<BarcodeFormat> barcodeFormats;

  /// Param that handle whether we need to save image from camera preview or make a snapshot while snapping barcode
  /// image uri will be returned in [BarcodeScanningResult.snappedImage] property
  BarcodeImageGenerationType barcodeImageGenerationType =
      BarcodeImageGenerationType.NONE;

  BarcodeScannerConfiguration(
      {this.topBarBackgroundColor,
      this.topBarButtonsColor,
      this.cameraOverlayColor,
      this.finderLineColor,
      this.finderTextHintColor,
      this.bottomButtonsActiveColor,
      this.bottomButtonsInactiveColor,
      this.cancelButtonTitle,
      this.enableCameraButtonTitle,
      this.enableCameraExplanationText,
      this.finderTextHint,
      this.flashButtonTitle,
      this.finderAspectRatio,
      this.finderLineWidth,
      this.successBeepEnabled,
      this.flashEnabled,
      this.orientationLockMode,
      this.barcodeFormats,
      this.cancelButtonHidden,
      this.barcodeImageGenerationType});

  factory BarcodeScannerConfiguration.fromJson(Map<String, dynamic> json) =>
      _$BarcodeScannerConfigurationFromJson(json);

  Map<String, dynamic> toJson() => _$BarcodeScannerConfigurationToJson(this);
}

enum BarcodeImageGenerationType { NONE, VIDEO_FRAME, CAPTURED_IMAGE }

@JsonSerializable(nullable: true, includeIfNull: false, explicitToJson: true)
class DetectBarcodeFromImageParams {
  ///file or content uri
  Uri uri;

  /// Enable more sensitive barcode detection.
  /// If set to true - will detect more barcodes.
  /// As a downside, might return false positives barcodes.
  /// Default is FALSE.
  bool enableHighSensitivityMode;

  /// Format of barcode which  need to recognize
  List<BarcodeFormat> barcodeFormats;

  DetectBarcodeFromImageParams(this.uri, this.barcodeFormats,
      this.enableHighSensitivityMode);

  factory DetectBarcodeFromImageParams.fromJson(Map<String, dynamic> json) =>
      _$DetectBarcodeFromImageParamsFromJson(json);

  Map<String, dynamic> toJson() => _$DetectBarcodeFromImageParamsToJson(this);
}

@JsonSerializable(
    nullable: true,
    createFactory: true,
    includeIfNull: false,
    explicitToJson: true)
class BarcodeFormattedData {

  String title;
  String subtitle;

  BarcodeFormattedData({this.title, this.subtitle});

  factory BarcodeFormattedData.fromJson(Map<String, dynamic> json) =>
      _$BarcodeFormattedDataFromJson(json);

  Map<String, dynamic> toJson() => _$BarcodeFormattedDataToJson(this);
}

typedef BarcodeDataFormatter = Future<BarcodeFormattedData> Function(BarcodeItem barcodeItem);

@JsonSerializable(nullable: true, includeIfNull: false, explicitToJson: true)
class BatchBarcodeScannerConfiguration {
  /// Allows to change data representation for scanned list items
  @JsonKey(ignore: true)
  BarcodeDataFormatter barcodeFormatter;

  /// Background color of the top toolbar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarBackgroundColor;

  /// Color of the titles of all buttons in the top toolbar.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarButtonsColor;

  /// Color buttons inactive color
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color topBarButtonsInactiveColor;

  /// Color of barcodes count text
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color barcodesCountTextColor;

  /// Color of barcodes details action
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color detailsActionColor;

  /// Color of barcodes details background
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color detailsBackgroundColor;

  /// Color of barcodes details background
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color detailsPrimaryColor;

  /// Background color outside of the finder window.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color cameraOverlayColor;

  /// Color of the finder window's outline.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color finderLineColor;

  /// Color of the text hint under the finder window.
  @JsonKey(fromJson: colorFromString, toJson: toColorString)
  Color finderTextHintColor;

  bool cancelButtonHidden;

  /// Title of the cancel button.
  String cancelButtonTitle;

  /// Title of the button that opens the screen where the user can allow
  /// the usage of the camera by the app.
  String enableCameraButtonTitle;

  /// Text that will be displayed when the app
  /// is not allowed to use the camera, prompting the user
  /// to enable the usage of the camera.
  String enableCameraExplanationText;

  /// Text hint shown under the finder window.
  String finderTextHint;

  /// Text of submit button.
  String submitButtonTitle;

  /// Text of clear button.
  String clearButtonTitle;

  /// Text of count text.
  String barcodesCountText;

  /// Text of fetch state text.
  String fetchingStateText;

  /// Text when there no scanned barcodes yet.
  String noBarcodesTitle;

  /// Aspect ratio of finder view.
  FinderAspectRatio finderAspectRatio;

  /// Thickness of the finder window's outline.
  int finderLineWidth;

  /// Controls whether to play a beep sound after a successful detection.
  /// Default value is TRUE.
  bool successBeepEnabled;

  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool flashEnabled;

  /// Orientation lock mode of the camera: PORTRAIT or LANDSCAPE.
  /// By default the camera orientation is not locked.

  CameraOrientationMode orientationLockMode;

  /// Format of barcode which  need to recognize
  List<BarcodeFormat> barcodeFormats;

  BatchBarcodeScannerConfiguration({this.topBarBackgroundColor,
    this.topBarButtonsColor,
    this.topBarButtonsInactiveColor,
    this.barcodeFormatter,
    this.barcodesCountTextColor,
    this.detailsActionColor,
    this.detailsBackgroundColor,
    this.detailsPrimaryColor,
    this.cameraOverlayColor,
    this.finderLineColor,
    this.finderTextHintColor,
    this.cancelButtonHidden,
    this.cancelButtonTitle,
    this.enableCameraButtonTitle,
    this.enableCameraExplanationText,
    this.finderTextHint,
    this.submitButtonTitle,
    this.clearButtonTitle,
    this.barcodesCountText,
    this.fetchingStateText,
    this.noBarcodesTitle,
    this.finderAspectRatio,
    this.finderLineWidth,
    this.successBeepEnabled,
    this.flashEnabled,
    this.orientationLockMode,
    this.barcodeFormats});

  factory BatchBarcodeScannerConfiguration.fromJson(
      Map<String, dynamic> json) =>
      _$BatchBarcodeScannerConfigurationFromJson(json);

  Map<String, dynamic> toJson() =>
      _$BatchBarcodeScannerConfigurationToJson(this);
}
